"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.shorthands = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const patching_apply_patch_1 = require("@pnpm/patching.apply-patch");
const cli_utils_1 = require("@pnpm/cli-utils");
const config_1 = require("@pnpm/config");
const pick_1 = __importDefault(require("ramda/src/pick"));
const render_help_1 = __importDefault(require("render-help"));
const tempy_1 = __importDefault(require("tempy"));
const error_1 = require("@pnpm/error");
const writePackage_1 = require("./writePackage");
const getPatchedDependency_1 = require("./getPatchedDependency");
const read_project_manifest_1 = require("@pnpm/read-project-manifest");
function rcOptionsTypes() {
    return (0, pick_1.default)([], config_1.types);
}
exports.rcOptionsTypes = rcOptionsTypes;
function cliOptionsTypes() {
    return { ...rcOptionsTypes(), 'edit-dir': String, 'ignore-existing': Boolean };
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.shorthands = {
    d: '--edit-dir',
};
exports.commandNames = ['patch'];
function help() {
    return (0, render_help_1.default)({
        description: 'Prepare a package for patching',
        descriptionLists: [{
                title: 'Options',
                list: [
                    {
                        description: 'The package that needs to be modified will be extracted to this directory',
                        name: '--edit-dir',
                    },
                    {
                        description: 'Ignore existing patch files when patching',
                        name: '--ignore-existing',
                    },
                ],
            }],
        url: (0, cli_utils_1.docsUrl)('patch'),
        usages: ['pnpm patch <pkg name>@<version>'],
    });
}
exports.help = help;
async function handler(opts, params) {
    if (opts.editDir && fs_1.default.existsSync(opts.editDir) && fs_1.default.readdirSync(opts.editDir).length > 0) {
        throw new error_1.PnpmError('PATCH_EDIT_DIR_EXISTS', `The target directory already exists: '${opts.editDir}'`);
    }
    if (!params[0]) {
        throw new error_1.PnpmError('MISSING_PACKAGE_NAME', '`pnpm patch` requires the package name');
    }
    const editDir = opts.editDir ?? tempy_1.default.directory();
    const lockfileDir = opts.lockfileDir ?? opts.dir ?? process.cwd();
    const patchedDep = await (0, getPatchedDependency_1.getPatchedDependency)(params[0], {
        lockfileDir,
        modulesDir: opts.modulesDir,
        virtualStoreDir: opts.virtualStoreDir,
    });
    await (0, writePackage_1.writePackage)(patchedDep, editDir, opts);
    if (!opts.ignoreExisting) {
        let rootProjectManifest = opts.rootProjectManifest;
        if (!opts.sharedWorkspaceLockfile) {
            const { manifest } = await (0, read_project_manifest_1.tryReadProjectManifest)(lockfileDir);
            if (manifest) {
                rootProjectManifest = manifest;
            }
        }
        if (rootProjectManifest?.pnpm?.patchedDependencies) {
            tryPatchWithExistingPatchFile({
                patchedDep,
                patchedDir: editDir,
                patchedDependencies: rootProjectManifest.pnpm.patchedDependencies,
                lockfileDir,
            });
        }
    }
    return `You can now edit the following folder: ${editDir}

Once you're done with your changes, run "pnpm patch-commit '${editDir}'"`;
}
exports.handler = handler;
function tryPatchWithExistingPatchFile({ patchedDep, patchedDir, patchedDependencies, lockfileDir, }) {
    if (!patchedDep.alias || !patchedDep.pref) {
        return;
    }
    const existingPatchFile = patchedDependencies[`${patchedDep.alias}@${patchedDep.pref}`];
    if (!existingPatchFile) {
        return;
    }
    const existingPatchFilePath = path_1.default.resolve(lockfileDir, existingPatchFile);
    if (!fs_1.default.existsSync(existingPatchFilePath)) {
        throw new error_1.PnpmError('PATCH_FILE_NOT_FOUND', `Unable to find patch file ${existingPatchFilePath}`);
    }
    (0, patching_apply_patch_1.applyPatchToDir)({ patchedDir, patchFilePath: existingPatchFilePath });
}
//# sourceMappingURL=patch.js.map